(* C example by Martin Boehme,
   Ported to Free Pascal by The Creative CAT *)
Program PacketRead4;
uses avutil, avcodec, avformat, AVRead;

procedure FatalError(s : string);
begin
  writeln(stderr, s);
  halt(-1)
end;

function fnval(s : string) : longint;
var i, j : integer;
begin
  if s >= '0' then val(s, i, j) else j := -1;
  if j = 0 then fnval := i else fnval := 0
end;

procedure SaveFrame(pFrame : PAVFrame; width, height, iFrame : longint);
var
  pFile : file;
  t : text;
  filename : string;
  y : longint;
  dummy : longint;
  b : PByte;

  function fnstr(i : longint) : string;
  var s : string;
  begin
    str(i, s);
    fnstr := s
  end;

  function ReplaceWithNum(s : string; n : longint) : string;
  var i : integer;
  begin
    i := pos('%d', s);
    delete(s, i, 2);
    insert(fnstr(n), s, i);
    ReplaceWithNum := s
  end;

begin
(* Open file *)
  filename := ReplaceWithNum('frame%d.ppm', iFrame);
  assign(t, filename);
  rewrite(t);
(* write header *)
  writeln(t, 'P6');
  writeln(t, width, ' ', height);
  writeln(t, 255);
  close(t);
(* write binary *)
  assign(pFile, filename);
  reset(pFile, 1);
  seek(pFile, filesize(pFile));			// points eof

  for y := 0 to pred(height) do begin   // Write pixel data
    b := pFrame^.data[0] + y * pFrame^.linesize[0];  (* ptr arithmetic *)
    blockwrite(pFile, b^, width*3, dummy)
  end;

  close(pFile)
end;

begin { main }
  writeln('ffmpeg test for Free Pascal');
  if paramcount = 0 then FatalError('file name required');

  decoder.init(paramstr(1), PIX_FMT_RGB24, -1);
  decoder.FormatCtxLocalDump(stdout);
  decoder.videoStreamParamLocalDump(stdout);

  (* Read frames and save first few frames to disk *)
  with decoder do begin
    GetNextFrame;
    while availFrame do begin
      ConvertFrame;
	  writeln(isKeyFrame, ' ', pred(FrameNum), ' ', FramePTS, ' ', FrameDispNum, ' ', FrameCodedNum);
      if FrameNum < fnval(paramstr(2)) then
		SaveFrame(pDestFrame, width, height, FrameNum); // Save the frame to disk
	  GetNextFrame
    end
  end;
  writeln('Total ', decoder.FrameNum, ' frames');

  decoder.done
end.
